@@ -86,6 +86,20 @@ awn_config_list_type_get_type(void) {
     }
   return enum_type_id;
 }
+GType
+awn_config_backend_get_type(void) {
+  static GType enum_type_id = 0;
+  if (G_UNLIKELY (!enum_type_id))
+    {
+      static const GEnumValue values[] = {
+        { AWN_CONFIG_CLIENT_GCONF, "AWN_CONFIG_CLIENT_GCONF", "gconf" },
+        { AWN_CONFIG_CLIENT_GKEYFILE, "AWN_CONFIG_CLIENT_GKEYFILE", "gkeyfile" },
+        { 0, NULL, NULL }
+      };
+      enum_type_id = g_enum_register_static("AwnConfigBackend", values);
+    }
+  return enum_type_id;
+}
 /* enumerations from "awn-effects.h" */
 #include "awn-effects.h"
 GType
@@ -168,6 +182,27 @@ awn_vfs_monitor_type_get_type(void) {
     }
   return enum_type_id;
 }
+/* enumerations from "awn-effects-shared.h" */
+#include "awn-effects-shared.h"
+GType
+awn_effect_priority_get_type(void) {
+  static GType enum_type_id = 0;
+  if (G_UNLIKELY (!enum_type_id))
+    {
+      static const GEnumValue values[] = {
+        { AWN_EFFECT_PRIORITY_HIGHEST, "AWN_EFFECT_PRIORITY_HIGHEST", "highest" },
+        { AWN_EFFECT_PRIORITY_HIGH, "AWN_EFFECT_PRIORITY_HIGH", "high" },
+        { AWN_EFFECT_PRIORITY_ABOVE_NORMAL, "AWN_EFFECT_PRIORITY_ABOVE_NORMAL", "above-normal" },
+        { AWN_EFFECT_PRIORITY_NORMAL, "AWN_EFFECT_PRIORITY_NORMAL", "normal" },
+        { AWN_EFFECT_PRIORITY_BELOW_NORMAL, "AWN_EFFECT_PRIORITY_BELOW_NORMAL", "below-normal" },
+        { AWN_EFFECT_PRIORITY_LOW, "AWN_EFFECT_PRIORITY_LOW", "low" },
+        { AWN_EFFECT_PRIORITY_LOWEST, "AWN_EFFECT_PRIORITY_LOWEST", "lowest" },
+        { 0, NULL, NULL }
+      };
+      enum_type_id = g_enum_register_static("AwnEffectPriority", values);
+    }
+  return enum_type_id;
+}
 
 /* Generated data ends here */
 
